@@ -1,4 +1,5 @@
-{-# OPTIONS_GHC -fffi -cpp #-}
+{-# LANGUAGE CPP, ForeignFunctionInterface #-}
+
 -- |
 -- Module:    Data.Digest.SHA256
 -- Copyright: Zooko O'Whielacronx
@@ -13,16 +14,21 @@
 
 module Crypt.SHA256 ( sha256sum ) where
 
-import FastPackedString ( PackedString, unsafeWithCStringLenPS )
 import Foreign
 import Foreign.C.Types
 import Numeric (showHex)
 import Foreign.C.String ( withCString )
+#if __GLASGOW_HASKELL__ > 606
+import Data.ByteString.Unsafe (unsafeUseAsCStringLen)
+#else
+import Data.ByteString.Base (unsafeUseAsCStringLen)
+#endif
+import qualified Data.ByteString as B
 
-sha256sum :: PackedString -> String
+sha256sum :: B.ByteString -> String
 sha256sum p = unsafePerformIO $
               withCString (take 64 $ repeat 'x') $ \digestCString ->
-              unsafeWithCStringLenPS p $ \(ptr,n) ->
+              unsafeUseAsCStringLen p $ \(ptr,n) ->
               do let digest = castPtr digestCString :: Ptr Word8
                  c_sha256 ptr (fromIntegral n) digest
                  go digest 0 []
