@@ -1,4 +1,4 @@
-{-# OPTIONS_GHC -cpp -fffi #-}
+{-# LANGUAGE CPP, ForeignFunctionInterface #-}
 
 module Darcs.External (
     backupByRenaming, backupByCopying,
@@ -16,7 +16,7 @@ module Darcs.External (
   ) where
 
 import Data.List ( intersperse )
-import Control.Monad ( liftM, when, zipWithM_ )
+import Control.Monad ( when, zipWithM_ )
 import System.Exit ( ExitCode(..) )
 import System.Environment ( getEnv )
 import System.Cmd ( system )
@@ -44,18 +44,22 @@ import Foreign.Ptr ( Ptr )
 import Foreign.Marshal.Alloc (allocaBytes)
 import Autoconf ( use_color )
 #endif
-import Workaround ( createLink, createDirectoryIfMissing )
+import System.Posix.Files ( createLink )
+import System.Directory ( createDirectoryIfMissing )
+import System.FilePath.Posix ( (</>), takeDirectory, normalise )
 
 import Darcs.Flags ( DarcsFlag( SignAs, Sign, SignSSL, NoLinks,
                                 Verify, VerifySSL ) )
 import Darcs.RepoPath ( AbsolutePath, toFilePath )
 import Darcs.Utils ( withCurrentDirectory, breakCommand, get_viewer, ortryrunning, )
-import Darcs.Progress ( withoutProgress, progressList, debugMessage )
-import FastPackedString ( PackedString, readFilePS, gzReadFilePS, writeFilePS,
-                          hPutPS, unpackPS, linesPS, unlinesPS,
-                          lengthPS, takePS, dropPS, packString,
-                          nullPS, nilPS, concatPS, hGetContentsPS
-                        )
+import Progress ( withoutProgress, progressList, debugMessage )
+
+import ByteStringUtils (gzReadFilePS, linesPS, unlinesPS)
+import qualified Data.ByteString as B (ByteString, empty, null, readFile -- ratify readFile: Just an import from ByteString
+            ,hGetContents, writeFile, hPut, length -- ratify hGetContents: importing from ByteString
+            ,take, concat, drop, isPrefixOf)
+import qualified Data.ByteString.Char8 as BC (unpack, pack)
+
 import Darcs.Lock ( withTemp, withOpenTemp, tempdir_loc, canonFilename, writeDocBinFile,
                     removeFileMayNotExist, )
 import CommandLine ( parseCmd, addUrlencoded )
@@ -64,7 +68,6 @@ import URL ( copyUrl, copyUrlFirst, waitUrl )
 import Ssh ( getSSH, copySSH, copySSHs, SSHCmd(..) )
 import URL ( Cachable(..) )
 import Exec ( exec, Redirect(..), withoutNonBlock )
-import UglyFileName ( fn2fp, fp2fn, norm_path )
 import Darcs.URL ( is_file, is_url, is_ssh )
 import Darcs.Utils ( catchall )
 import Printer ( Doc, Printers, putDocLnWith, hPutDoc, hPutDocLn, hPutDocWith, ($$), (<+>), renderPS,
@@ -84,9 +87,8 @@ backupByCopying = backupBy copy
   copy x y = do
     isD <- doesDirectoryExist x
     if isD then do createDirectory y
-                   cloneTree (do_norm x) (do_norm y)
+                   cloneTree (normalise x) (normalise y)
            else copyFile x y
-  do_norm f = fn2fp $ norm_path $ fp2fn f
 
 backupBy :: (FilePath -> FilePath -> IO ()) -> FilePath -> IO ()
 backupBy backup f =
@@ -103,12 +105,12 @@ backupBy backup f =
              where next = f ++ suffix
                    suffix = "-darcs-backup" ++ show i
 
-fetchFilePS :: String -> Cachable -> IO PackedString
-fetchFilePS fou _ | is_file fou = readFilePS fou
+fetchFilePS :: String -> Cachable -> IO B.ByteString
+fetchFilePS fou _ | is_file fou = B.readFile fou
 fetchFilePS fou cache = withTemp $ \t -> do copyFileOrUrl [] fou t cache
-                                            readFilePS t
+                                            B.readFile t
 
-gzFetchFilePS :: String -> Cachable -> IO PackedString
+gzFetchFilePS :: String -> Cachable -> IO B.ByteString
 gzFetchFilePS fou _ | is_file fou = gzReadFilePS fou
 gzFetchFilePS fou cache = withTemp $ \t-> do copyFileOrUrl [] fou t cache
                                              gzReadFilePS t
@@ -133,17 +135,16 @@ clonePaths source dest = mapM_ (clonePath source dest)
 
 clonePath :: FilePath -> FilePath -> FilePath -> IO ()
 clonePath source dest path
- = do let source' = source ++ "/" ++ path
-          dest' = dest ++ "/" ++ path
+ = do let source' = source </> path
+          dest' = dest </> path
       fs <- getSymbolicLinkStatus source'
       if isDirectory fs then do
           createDirectoryIfMissing True dest'
        else if isRegularFile fs then do
-          createDirectoryIfMissing True (dest ++ "/" ++ basename path)
+          createDirectoryIfMissing True (dest </> takeDirectory path)
           cloneFile source' dest'
        else fail ("clonePath: Bad file " ++ source')
-   `catch` fail ("clonePath: Bad file " ++ source ++ "/" ++ path)
- where basename = reverse . dropWhile ('/' /=) . dropWhile ('/' ==) . reverse
+   `catch` fail ("clonePath: Bad file " ++ source </> path)
 
 clonePartialsTree :: FilePath -> FilePath -> [FilePath] -> IO ()
 clonePartialsTree source dest = mapM_ (clonePartialTree source dest)
@@ -151,9 +152,8 @@ clonePartialsTree source dest = mapM_ (clonePartialTree source dest)
 clonePartialTree :: FilePath -> FilePath -> FilePath -> IO ()
 clonePartialTree source dest "" = cloneTree source dest
 clonePartialTree source dest pref
- = do createDirectoryIfMissing True (dest ++ "/" ++ basename pref)
-      cloneSubTree (source ++ "/" ++ pref) (dest ++ "/" ++ pref)
- where basename = reverse . dropWhile ('/' /=) . dropWhile ('/' ==) . reverse
+ = do createDirectoryIfMissing True (dest </> takeDirectory pref)
+      cloneSubTree (source </> pref) (dest </> pref)
 
 cloneTree :: FilePath -> FilePath -> IO ()
 cloneTree = cloneTreeExcept []
@@ -164,8 +164,8 @@ cloneTreeExcept except source dest =
     if isDirectory fs then do
         fps <- getDirectoryContents source
         let fps' = filter (`notElem` (".":"..":except)) fps
-            mk_source fp = source ++ "/" ++ fp
-            mk_dest   fp = dest   ++ "/" ++ fp
+            mk_source fp = source </> fp
+            mk_dest   fp = dest   </> fp
         zipWithM_ cloneSubTree (map mk_source fps') (map mk_dest fps')
      else fail ("cloneTreeExcept: Bad source " ++ source)
    `catch` fail ("cloneTreeExcept: Bad source " ++ source)
@@ -177,8 +177,8 @@ cloneSubTree source dest =
         createDirectory dest
         fps <- getDirectoryContents source
         let fps' = filter (`notElem` [".", ".."]) fps
-            mk_source fp = source ++ "/" ++ fp
-            mk_dest   fp = dest   ++ "/" ++ fp
+            mk_source fp = source </> fp
+            mk_dest   fp = dest   </> fp
         zipWithM_ cloneSubTree (map mk_source fps') (map mk_dest fps')
      else if isRegularFile fs then do
         cloneFile source dest
@@ -193,7 +193,7 @@ cloneFile = copyFile
 maybeURLCmd :: String -> String -> IO(Maybe(String))
 maybeURLCmd what url =
   do let prot = map toUpper $ takeWhile (/= ':') url
-     liftM Just (getEnv ("DARCS_" ++ what ++ "_" ++ prot))
+     fmap Just (getEnv ("DARCS_" ++ what ++ "_" ++ prot))
              `catch` \_ -> return Nothing
 
 speculateRemote :: String -> FilePath -> IO () -- speculations are always Cachable
@@ -421,7 +421,8 @@ sendEmailDoc f t s cc scmd mbundle body =
         hClose hat
         let ftable = [ ('t',addressOnly t),('c',cc),('f',f),('s',s) ] ++ ftable'
         r <- execSendmail ftable scmd fn
-        when (r /= ExitSuccess) $ fail ("failed to send mail to: " ++ t
+        when (r /= ExitSuccess) $ fail ("failed to send mail to: "
+                                       ++ t ++ cc_list cc
                                        ++ "\nPerhaps sendmail is not configured.")
   else if have_mapi then do
      r <- withCString t $ \tp ->
@@ -442,7 +443,10 @@ sendEmailDoc f t s cc scmd mbundle body =
           ('<':a2) -> takeWhile (/= '>') a2
           _        -> a
 
-resendEmail :: String -> String -> PackedString -> IO ()
+        cc_list [] = []
+        cc_list c = " and cc'ed " ++ c
+
+resendEmail :: String -> String -> B.ByteString -> IO ()
 resendEmail "" _ _ = return ()
 resendEmail t scmd body =
   case (have_sendmail || scmd /= "", have_mapi) of
@@ -458,22 +462,22 @@ resendEmail t scmd body =
      when (r /= ExitSuccess) $ fail ("failed to send mail to: " ++ t)
    (_, True) -> fail "Don't know how to resend email with MAPI"
    _ -> fail $ "no mail facility (sendmail or mapi) located at configure time (use the sendmail-command option)!"
-  where br = packString "\r"
-        darcsurl = packString "DarcsURL:"
-        content = packString "Content-"
-        from_start = packString "From:"
-        subject_start = packString "Subject:"
+  where br            = BC.pack "\r"
+        darcsurl      = BC.pack "DarcsURL:"
+        content       = BC.pack "Content-"
+        from_start    = BC.pack "From:"
+        subject_start = BC.pack "Subject:"
         fixit (l:ls)
-         | nullPS l = packedString nilPS $$ vcat (map packedString ls)
-         | l == br = packedString nilPS $$ vcat (map packedString ls)
-         | takePS 9 l == darcsurl || takePS 8 l == content
+         | B.null l = packedString B.empty $$ vcat (map packedString ls)
+         | l == br = packedString B.empty $$ vcat (map packedString ls)
+         | B.take 9 l == darcsurl || B.take 8 l == content
             = packedString l $$ fixit ls
          | otherwise = fixit ls
         fixit [] = empty
-        find_from (l:ls) | takePS 5 l == from_start = unpackPS l
+        find_from (l:ls) | B.take 5 l == from_start = BC.unpack l
                          | otherwise = find_from ls
         find_from [] = "From: unknown"
-        find_subject (l:ls) | takePS 8 l == subject_start = unpackPS l
+        find_subject (l:ls) | B.take 8 l == subject_start = BC.unpack l
                             | otherwise = find_subject ls
         find_subject [] = "Subject: (no subject)"
 
@@ -504,8 +508,8 @@ c_send_email
 c_send_email = impossible
 #endif
 
-execPSPipe :: String -> [String] -> PackedString -> IO PackedString
-execPSPipe c args ps = liftM renderPS
+execPSPipe :: String -> [String] -> B.ByteString -> IO B.ByteString
+execPSPipe c args ps = fmap renderPS
                      $ execDocPipe c args
                      $ packedString ps
 
@@ -517,7 +521,7 @@ execDocPipe c args instr = withoutProgress $
        forkIO ((hGetContents e >>= -- ratify hGetContents: it's immediately consumed
                 hPutStr stderr)
                `finally` putMVar mvare ())
-       out <- hGetContentsPS o
+       out <- B.hGetContents o
        rval <- waitForProcess pid
        takeMVar mvare
        case rval of
@@ -535,7 +539,7 @@ execPipeIgnoreError c args instr = withoutProgress $
        forkIO ((hGetContents e >>= -- ratify hGetContents: it's immediately consumed
                 hPutStr stderr)
                `finally` putMVar mvare ())
-       out <- hGetContentsPS o
+       out <- B.hGetContents o
        waitForProcess pid
        takeMVar mvare
        return $ packedString out
@@ -555,28 +559,28 @@ signSSL idfile t =
     withTemp $ \cert -> do
     opensslPS ["req", "-new", "-key", idfile,
                "-outform", "PEM", "-days", "365"]
-                (packString "\n\n\n\n\n\n\n\n\n\n\n")
+                (BC.pack "\n\n\n\n\n\n\n\n\n\n\n")
                 >>= opensslPS ["x509", "-req", "-extensions",
                                "v3_ca", "-signkey", idfile,
                                "-outform", "PEM", "-days", "365"]
                 >>= opensslPS ["x509", "-outform", "PEM"]
-                >>= writeFilePS cert
+                >>= B.writeFile cert
     opensslDoc ["smime", "-sign", "-signer", cert,
                 "-inkey", idfile, "-noattr", "-text"] t
     where opensslDoc = execDocPipe "openssl"
           opensslPS = execPSPipe "openssl"
 
 
-verifyPS :: [DarcsFlag] -> PackedString -> IO (Maybe PackedString)
+verifyPS :: [DarcsFlag] -> B.ByteString -> IO (Maybe B.ByteString)
 verifyPS [] ps = return $ Just ps
 verifyPS (Verify pks:_) ps = verifyGPG pks ps
 verifyPS (VerifySSL auks:_) ps = verifySSL auks ps
 verifyPS (_:os) ps = verifyPS os ps
 
-verifyGPG :: AbsolutePath -> PackedString -> IO (Maybe PackedString)
+verifyGPG :: AbsolutePath -> B.ByteString -> IO (Maybe B.ByteString)
 verifyGPG goodkeys s =
     withOpenTemp $ \(th,tn) -> do
-      hPutPS th s
+      B.hPut th s
       hClose th
       rval <- exec "gpg"  ["--batch","--no-default-keyring",
                            "--keyring",fix_path $ toFilePath goodkeys, "--verify"]
@@ -586,12 +590,12 @@ verifyGPG goodkeys s =
           _ -> return Nothing
       where gpg_fixed_s = let
                 not_begin_signature x =
-                    x /= packString "-----BEGIN PGP SIGNED MESSAGE-----"
+                    x /= BC.pack "-----BEGIN PGP SIGNED MESSAGE-----"
                     &&
-                    x /= packString "-----BEGIN PGP SIGNED MESSAGE-----\r"
+                    x /= BC.pack "-----BEGIN PGP SIGNED MESSAGE-----\r"
                 in unlinesPS $ map fix_line $ tail $ dropWhile not_begin_signature $ linesPS s
-            fix_line x | lengthPS x < 3 = x
-                       | takePS 3 x == packString "- -" = dropPS 2 x
+            fix_line x | B.length x < 3 = x
+                       | BC.pack "- -" `B.isPrefixOf` x = B.drop 2 x
                        | otherwise = x
 #if defined(WIN32)
             fix_sep c | c=='/' = '\\'   | otherwise = c
@@ -600,28 +604,28 @@ verifyGPG goodkeys s =
             fix_path p = p
 #endif
 
-verifySSL :: AbsolutePath -> PackedString -> IO (Maybe PackedString)
+verifySSL :: AbsolutePath -> B.ByteString -> IO (Maybe B.ByteString)
 verifySSL goodkeys s = do
     certdata <- opensslPS ["smime", "-pk7out"] s
                 >>= opensslPS ["pkcs7", "-print_certs"]
     cruddy_pk <- opensslPS ["x509", "-pubkey"] certdata
-    let key_used = concatPS $ tail $
-                   takeWhile (/= packString"-----END PUBLIC KEY-----")
+    let key_used = B.concat $ tail $
+                   takeWhile (/= BC.pack"-----END PUBLIC KEY-----")
                            $ linesPS cruddy_pk
-        in do allowed_keys <- linesPS `liftM` readFilePS (toFilePath goodkeys)
+        in do allowed_keys <- linesPS `fmap` B.readFile (toFilePath goodkeys)
               if not $ key_used `elem` allowed_keys
                 then return Nothing -- Not an allowed key!
                 else withTemp $ \cert ->
                      withTemp $ \on ->
                      withOpenTemp $ \(th,tn) -> do
-                     hPutPS th s
+                     B.hPut th s
                      hClose th
-                     writeFilePS cert certdata
+                     B.writeFile cert certdata
                      rval <- exec "openssl" ["smime", "-verify", "-CAfile",
                                              cert, "-certfile", cert]
                                              (File tn, File on, Null)
                      case rval of
-                       ExitSuccess -> Just `liftM` readFilePS on
+                       ExitSuccess -> Just `fmap` B.readFile on
                        _ -> return Nothing
     where opensslPS = execPSPipe "openssl"
 
@@ -643,10 +647,10 @@ can_execute (arg,expected_return_value) exe = do
 getTermNColors :: IO Int
 
 #ifdef HAVE_TERMINFO
-getTermNColors = do                          
-  t <- setupTermFromEnv                      
+getTermNColors = do
+  t <- setupTermFromEnv
   return $ case getCapability t $ tiGetNum "colors" of
-    Nothing -> (-1)                          
+    Nothing -> (-1)
     Just x -> x
 
 #elif HAVE_CURSES
@@ -686,6 +690,9 @@ viewDocWith pr msg = do
              pipeDocToPager viewer [] pr msg
                `ortryrunning` pipeDocToPager  "less" [] pr msg
                `ortryrunning` pipeDocToPager  "more" [] pr msg
+#ifdef WIN32
+               `ortryrunning` pipeDocToPager  "more.com" [] pr msg
+#endif
                `ortryrunning` pipeDocToPager "" [] pr msg
      else pipeDocToPager "" [] pr msg
   return ()
@@ -704,12 +711,12 @@ pipeDocToPager c args pr inp = withoutNonBlock $ withoutProgress $ do
   bracket (openBinaryTempFile tmp "darcs-pager") cleanup $ \(fn,fh) ->
     do hPutDocWith pr fh inp
        hClose fh
-       h <- openBinaryFile fn ReadMode
-       x <- do pid <- runProcess c args Nothing Nothing (Just h) Nothing Nothing
-               waitForProcess pid
-       when (x == ExitFailure 127) $
-            putStrLn $ "Command not found:\n   "++ show (c:args)
-       return x
+       bracket (openBinaryFile fn ReadMode) hClose $ \h ->
+         do x <- do pid <- runProcess c args Nothing Nothing (Just h) Nothing Nothing
+                    waitForProcess pid
+            when (x == ExitFailure 127) $
+                 putStrLn $ "Command not found:\n   "++ show (c:args)
+            return x
   where
     cleanup (f,h) = do try $ hClose h
                        removeFileMayNotExist f
