@@ -1,4 +1,6 @@
 {-# OPTIONS_GHC -cpp #-}
+{-# LANGUAGE CPP #-}
+
 -- Copyright (C) 2007 Eric Kow
 --
 -- This program is free software; you can redistribute it and/or modify
@@ -20,21 +22,22 @@ module Darcs.RepoPath ( AbsolutePath, makeAbsolute, ioAbsolute, rootDirectory,
                         SubPath, makeSubPathOf, simpleSubPath,
                         AbsolutePathOrStd,
                         makeAbsoluteOrStd, ioAbsoluteOrStd, useAbsoluteOrStd,
-                        makeRelative, sp2fn,
+                        AbsoluteOrRemotePath, ioAbsoluteOrRemote, isRemote,
+                        sp2fn,
                         FilePathOrURL(..), FilePathLike(toFilePath),
                         getCurrentDirectory, setCurrentDirectory
                       ) where
 
-import Data.List ( isPrefixOf )
+import Data.List ( isPrefixOf, isSuffixOf )
 import Control.Exception ( bracket )
 
-import Darcs.URL ( is_absolute, is_relative )
+import Darcs.URL ( is_absolute, is_relative, is_ssh_nopath )
 import Autoconf ( path_separator )
 import qualified Workaround ( getCurrentDirectory )
 import qualified System.Directory ( setCurrentDirectory )
 import System.Directory ( doesDirectoryExist )
-import UglyFileName ( fn2fp, super_name, fp2fn, own_name, norm_path )
-import qualified Darcs.Patch.FileName as PatchFileName ( FileName, fp2fn )
+import qualified System.FilePath.Posix as FilePath
+import qualified Darcs.Patch.FileName as PatchFileName ( FileName, fp2fn, fn2fp )
 
 class FilePathOrURL a where
  {-# INLINE toPath #-}
@@ -43,17 +46,13 @@ class FilePathOrURL a where
 class FilePathOrURL a => FilePathLike a where
  {-# INLINE toFilePath #-}
  toFilePath :: a -> FilePath
- {-# INLINE toInternal #-}
- toInternal :: a -> FilePath
- toInternal = toFilePath
- {-# INLINE fromFilePath #-}
- fromFilePath :: FilePath -> a
 
 -- | Relative to the local darcs repository and normalized
 --   Note: these are understood not to have the dot in front
 newtype SubPath      = SubPath FilePath deriving (Eq, Ord)
 newtype AbsolutePath = AbsolutePath FilePath deriving (Eq, Ord)
 data AbsolutePathOrStd = AP AbsolutePath | APStd deriving (Eq, Ord)
+data AbsoluteOrRemotePath = AbsP AbsolutePath | RmtP String deriving (Eq, Ord)
 
 instance FilePathOrURL AbsolutePath where
  toPath (AbsolutePath x) = x
@@ -62,14 +61,19 @@ instance FilePathOrURL SubPath where
 instance CharLike c => FilePathOrURL [c] where
  toPath = toFilePath
 
+instance FilePathOrURL AbsoluteOrRemotePath where
+ toPath (AbsP a) = toPath a
+ toPath (RmtP r) = r
+
+instance FilePathOrURL PatchFileName.FileName where
+    toPath = PatchFileName.fn2fp
+instance FilePathLike PatchFileName.FileName where
+    toFilePath = PatchFileName.fn2fp
+
 instance FilePathLike AbsolutePath where
  toFilePath (AbsolutePath x) = x
- toInternal (AbsolutePath x) = x
- fromFilePath xs       = AbsolutePath xs
 instance FilePathLike SubPath where
  toFilePath (SubPath x) = x
- toInternal (SubPath x) = x
- fromFilePath = SubPath
 
 class CharLike c where
     toChar :: c -> Char
@@ -80,8 +84,6 @@ instance CharLike Char where
 
 instance CharLike c => FilePathLike [c] where
     toFilePath = map toChar
-    toInternal = map toChar
-    fromFilePath = map fromChar
 
 -- | Make the second path relative to the first, if possible
 makeSubPathOf :: AbsolutePath -> AbsolutePath -> Maybe SubPath
@@ -92,19 +94,9 @@ makeSubPathOf (AbsolutePath p1) (AbsolutePath p2) =
     else Nothing
 
 simpleSubPath :: FilePath -> Maybe SubPath
-simpleSubPath x | is_relative x = Just $ SubPath $ fn2fp $ norm_path $ fp2fn $ map cleanup x
+simpleSubPath x | is_relative x = Just $ SubPath $ FilePath.normalise $ map cleanup x
                 | otherwise = Nothing
 
-makeRelative :: AbsolutePath -> AbsolutePath -> FilePath
-makeRelative (AbsolutePath p1) (AbsolutePath p2) = mr p1 p2
-    where mr x y | x == y = "."
-          mr x y | takedir x == takedir y = mr (dropdir x) (dropdir y)
-          mr x y = add_dotdots x y
-          add_dotdots "" y = dropWhile  (=='/') y
-          add_dotdots x y = '.':'.':'/': add_dotdots (dropdir x) y
-          takedir = takeWhile (/='/')  . dropWhile (=='/')
-          dropdir = dropWhile (/='/') . dropWhile (=='/')
-
 -- | Interpret a possibly relative path wrt the current working directory
 ioAbsolute :: FilePath -> IO AbsolutePath
 ioAbsolute dir =
@@ -114,16 +106,18 @@ ioAbsolute dir =
          then bracket (setCurrentDirectory dir)
                       (const $ setCurrentDirectory $ toFilePath here)
                       (const getCurrentDirectory)
-         else let super_dir = (fn2fp . super_name . fp2fn) dir
-                  file = (fn2fp . own_name . fp2fn) dir
+         else let super_dir = case FilePath.takeDirectory dir of
+                                "" ->  "."
+                                d  -> d
+                  file = FilePath.takeFileName dir
               in do abs_dir <- ioAbsolute super_dir
                     return $ makeAbsolute abs_dir file
 
 makeAbsolute :: AbsolutePath -> FilePath -> AbsolutePath
 makeAbsolute a dir = if is_absolute dir
                      then AbsolutePath $
-                          slashes ++ (fn2fp $ norm_path $ fp2fn cleandir)
-                     else ma a $ fn2fp $ norm_path $ fp2fn cleandir
+                          slashes ++ FilePath.normalise cleandir
+                     else ma a $ FilePath.normalise cleandir
   where
     cleandir  = map cleanup dir
     slashes = norm_slashes $ takeWhile (== '/') cleandir
@@ -157,6 +151,20 @@ useAbsoluteOrStd :: (AbsolutePath -> IO a) -> IO a -> AbsolutePathOrStd -> IO a
 useAbsoluteOrStd _ f APStd = f
 useAbsoluteOrStd f _ (AP x) = f x
 
+ioAbsoluteOrRemote :: String -> IO AbsoluteOrRemotePath
+ioAbsoluteOrRemote p = do
+  isdir <- doesDirectoryExist p
+  if not isdir
+     then return $ RmtP $
+          case () of _ | is_ssh_nopath p    -> p++"."
+                       | "/" `isSuffixOf` p -> init p
+                       | otherwise          -> p
+     else AbsP `fmap` ioAbsolute p
+
+isRemote :: AbsoluteOrRemotePath -> Bool
+isRemote (RmtP _) = True
+isRemote _ = False
+
 takeDirectory :: AbsolutePath -> AbsolutePath
 takeDirectory (AbsolutePath x) =
     case reverse $ drop 1 $ dropWhile (/='/') $ reverse x of
@@ -170,6 +178,9 @@ instance Show SubPath where
 instance Show AbsolutePathOrStd where
     show (AP a) = show a
     show APStd = "standard input/output"
+instance Show AbsoluteOrRemotePath where
+    show (AbsP a) = show a
+    show (RmtP r) = show r
 
 cleanup :: Char -> Char
 cleanup '\\' | path_separator == '\\' = '/'
