@@ -1,4 +1,6 @@
 {-# OPTIONS_GHC -cpp #-}
+{-# LANGUAGE CPP #-}
+
 {-
 Copyright (C) 2004 David Roundy
 
@@ -18,6 +20,33 @@ the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 Boston, MA 02110-1301, USA.
 -}
 
+{-|
+
+  Path resolving:
+
+    * A URL contains the sequence @\":\/\/\"@.
+
+    * A local filepath does not contain colons, except
+      as second character (windows drives).
+
+    * A path that is neither a URL nor a local file
+      is an ssh-path.
+
+  Examples:
+
+  > /usr/repo/foo                 -- local file
+  > c:/src/darcs                  -- local file
+  > http://darcs.net/             -- URL
+  > peter@host:/path              -- ssh
+  > droundy@host:                 -- ssh
+  > host:/path                    -- ssh
+
+  This means that single-letter hosts in ssh-paths do not work,
+  unless a username is provided.
+
+  Perhaps ssh-paths should use @\"ssh:\/\/user\@host\/path\"@-syntax instead?
+-}
+
 module Darcs.URL (
     is_file, is_url, is_ssh, is_relative, is_absolute,
     is_ssh_nopath
@@ -25,29 +54,6 @@ module Darcs.URL (
 
 #include "impossible.h"
 
-{-
-  Path resolving:
-    * A URL contains the sequence "://".
-    * A local filepath does not contain colons, except
-      as second character (windows drives).
-    * A path that is neither a URL nor a local file
-      is an ssh-path.
-
-  Examples:
-    /usr/repo/foo                 - local file
-    c:/src/darcs                  - local file
-    http://darcs.net/             - URL
-    peter@host:/path              - ssh
-    droundy@host:                 - ssh
-    host:/path                    - ssh
-
-
-  This means that single-letter hosts in ssh-paths doesn't work,
-  unless a username is provided.
-
-  Perhaps ssh-paths should use "ssh://user@host/path"-syntax instead?
--}
-
 is_relative :: String -> Bool
 is_relative (_:':':_) = False
 is_relative f@(c:_) = is_file f && c /= '/' && c /= '~'
