@@ -1,4 +1,6 @@
 {-# OPTIONS_GHC -cpp #-}
+{-# LANGUAGE CPP #-}
+
 module HTTP( fetchUrl, postUrl, request_url, wait_next_url ) where
 
 import Darcs.Global ( debugFail )
@@ -12,6 +14,8 @@ import System.Environment ( getEnv )
 import System.IO.Error ( ioeGetErrorString )
 import System.IO.Unsafe ( unsafePerformIO )
 import Darcs.Global ( debugMessage )
+import qualified Data.ByteString as B
+import qualified Data.ByteString.Char8 as BC
 #endif
 
 fetchUrl :: String -> IO String
@@ -83,7 +87,7 @@ wait_next_url = do
   if null u
      then return ("", "No URL requested")
      else do writeIORef requestedUrl ("", "")
-             e <- (fetchUrl u >>= writeFile f >> return "") `catch` h
+             e <- (fetchUrl u >>= \s -> B.writeFile f (BC.pack s) >> return "") `catch` h
              return (u, e)
     where h = return . ioeGetErrorString
 
