@@ -1,4 +1,4 @@
-{-# OPTIONS_GHC -cpp -fffi #-}
+{-# LANGUAGE CPP, ForeignFunctionInterface #-}
 
 module Ssh ( grabSSH, runSSH, getSSH, copySSH, copySSHs, SSHCmd(..) ) where
 
@@ -15,7 +15,7 @@ import System.Random ( randomIO )
 #endif
 import System.IO ( Handle, hPutStr, hPutStrLn, hGetLine, hGetContents, hClose, hFlush )
 import System.IO.Unsafe ( unsafePerformIO )
-import System.Directory ( doesFileExist )
+import System.Directory ( doesFileExist, createDirectoryIfMissing )
 import Control.Monad ( when )
 import System.Process ( runInteractiveProcess )
 
@@ -23,14 +23,15 @@ import Data.Map ( Map, empty, insert, lookup )
 import Data.IORef ( IORef, newIORef, readIORef, modifyIORef )
 
 import Darcs.SignalHandler ( catchNonSignal )
-import Workaround ( createDirectoryIfMissing )
 import Darcs.Utils ( withCurrentDirectory, breakCommand, prettyException, catchall )
 import Darcs.Global ( atexit, sshControlMasterDisabled, darcsdir, withDebugMode )
 import Darcs.Lock ( withTemp, withOpenTemp, tempdir_loc, removeFileMayNotExist )
 import Exec ( exec, Redirects, Redirect(..), )
-import Darcs.Progress ( withoutProgress, debugMessage, debugFail, progressList )
+import Progress ( withoutProgress, debugMessage, debugFail, progressList )
+
+import qualified Data.ByteString as B (ByteString, hGet, writeFile, readFile)
+import qualified Data.ByteString.Char8 as BC (unpack)
 
-import FastPackedString ( PackedString, unpackPS, readFilePS, writeFilePS, hGetPS )
 #include "impossible.h"
 
 {-# NOINLINE sshConnections #-}
@@ -58,7 +59,7 @@ withSSHConnection x withconnection withoutconnection =
                            then return ()
                            else debugFail "Couldn't start darcs transfer-mode on server"
                        let c = C { inp = i, out = o, err = e,
-                                   deb = \s -> debugMessage ("with ssh "++uhost++": "++s) }
+                                   deb = \s -> debugMessage ("with ssh (transfer-mode) "++uhost++": "++s) }
                        modifyIORef sshConnections (insert url (Just c))
                        return $ Just c
                     `catchNonSignal`
@@ -86,7 +87,7 @@ cleanrepourl "" = ""
 cleanrepodir :: String -> String
 cleanrepodir = cleanrepourl . drop 1 . dropWhile (/= ':')
 
-grabSSH :: String -> Connection -> IO PackedString
+grabSSH :: String -> Connection -> IO B.ByteString
 grabSSH x c = do
                let dir = drop 1 $ dropWhile (/= ':') x
                    dd = darcsdir++"/"
@@ -107,7 +108,7 @@ grabSSH x c = do
                if l2 == "got "++file
                   then do showlen <- hGetLine (out c)
                           case reads showlen of
-                            [(len,"")] -> hGetPS (out c) len
+                            [(len,"")] -> B.hGet (out c) len
                             _ -> failwith "Couldn't get length"
                   else if l2 == "error "++file
                        then do e <- hGetLine (out c)
@@ -121,7 +122,7 @@ sshStdErrMode = withDebugMode $ \amdebugging ->
                 return $ if amdebugging then AsIs else Null
 
 copySSH :: String -> FilePath -> IO ()
-copySSH uRaw f = withSSHConnection uRaw (\c -> grabSSH uRaw c >>= writeFilePS f) $
+copySSH uRaw f = withSSHConnection uRaw (\c -> grabSSH uRaw c >>= B.writeFile f) $
               do let u = escape_dollar uRaw
                  stderr_behavior <- sshStdErrMode
                  r <- runSSH SCP u [] [u,f] (AsIs,AsIs,stderr_behavior)
@@ -136,7 +137,7 @@ copySSH uRaw f = withSSHConnection uRaw (\c -> grabSSH uRaw c >>= writeFilePS f)
 copySSHs :: String -> [String] -> FilePath -> IO ()
 copySSHs u ns d =
   withSSHConnection u (\c -> withCurrentDirectory d $
-                             mapM_ (\n -> grabSSH (u++"/"++n) c >>= writeFilePS n) $
+                             mapM_ (\n -> grabSSH (u++"/"++n) c >>= B.writeFile n) $
                              progressList "Copying via ssh" ns) $
   do let path = drop 1 $ dropWhile (/= ':') u
          host = takeWhile (/= ':') u
@@ -156,12 +157,12 @@ copySSHs u ns d =
                          then ["sftp doesn't expand ~, use path/ instead of ~/path/"]
                          else []
             when (r /= ExitSuccess) $ do
-                 outputPS <- readFilePS sftpoutput
+                 outputPS <- B.readFile sftpoutput
                  debugFail $ unlines $
                           ["(sftp) failed to fetch files.",
                            "source directory: " ++ path,
                            "source files:"] ++ files ++
-                          ["sftp output:",unpackPS outputPS] ++
+                          ["sftp output:",BC.unpack outputPS] ++
                           hint
 
 -- ---------------------------------------------------------------------
