@@ -1,4 +1,5 @@
-{-# OPTIONS_GHC -cpp -fffi #-}
+{-# LANGUAGE CPP, ForeignFunctionInterface #-}
+
 module URL ( copyUrl, copyUrlFirst, pipeliningEnabledByDefault,
              setDebugHTTP, setHTTPPipelining, waitUrl,
              Cachable(Cachable, Uncachable, MaxAge)
@@ -20,9 +21,12 @@ import Foreign.C.Types ( CInt )
 
 import Workaround ( renameFile )
 import Darcs.Global ( atexit )
-import Darcs.Progress ( debugFail, debugMessage )
+import Progress ( debugFail, debugMessage )
 import Darcs.Lock ( removeFileMayNotExist )
 
+import Numeric ( showHex )
+import System.Random ( randomRIO )
+
 #if defined(HAVE_CURL) || defined(HAVE_LIBWWW)
 import Foreign.C.String ( withCString, peekCString, CString )
 #elif defined(HAVE_HTTP)
@@ -44,7 +48,8 @@ data UrlState = UrlState { inProgress :: Map String ( FilePath
                                                     , Cachable
                                                     , (MVar String) )
                          , waitToStart :: Q String
-                         , pipeLength :: Int }
+                         , pipeLength :: Int
+                         , randomJunk :: String }
 
 data Q a = Q [a] [a]
 
@@ -107,8 +112,10 @@ urlChan = unsafePerformIO $ do
   return ch
 
 urlThread :: Chan UrlRequest -> IO ()
-urlThread ch = evalStateT urlThread' (UrlState Map.empty emptyQ 0)
-    where urlThread' = do
+urlThread ch = do junk <- flip showHex "" `fmap` randomRIO rrange
+                  evalStateT urlThread' (UrlState Map.empty emptyQ 0 junk)
+    where rrange = (0, 2^(128 :: Integer) :: Integer)
+          urlThread' = do
             empty <- liftIO $ isEmptyChan ch
             st <- get
             let l = pipeLength st
@@ -147,18 +154,20 @@ urlThread ch = evalStateT urlThread' (UrlState Map.empty emptyQ 0)
                      new_st = st { inProgress = Map.insert u e p
                                  , waitToStart = new_w }
                  case Map.lookup u p of
-                   Just (f', fs', c', v) ->
-                     if c /= c'
-                     then bug $ "URL.urlThread: same URLs with different parameters "++u++" "++show (c,c')
-                     else do when (u `elemQ` w && priority r == High) $ do
-                                  modify (\s -> s { waitToStart = pushQ u (deleteQ u w) })
-                                  dbg $ "Moving "++u++" to head of download queue."
-                             if f `notElem` (f':fs')
-                                then let new_p = Map.insert u (f', f:fs', c', v) p
-                                     in do modify (\s -> s { inProgress = new_p })
-                                           dbg "Adding new file to existing UrlRequest."
-                                else dbg "Ignoring UrlRequest of file that's already queued."
-                   _                -> put new_st
+                   Just (f', fs', c', v) -> do
+                     let new_c = minCachable c c'
+                     when (c /= c') $ let new_p = Map.insert u (f', fs', new_c, v) p
+                                      in do modify (\s -> s { inProgress = new_p })
+                                            dbg $ "Changing "++u++" request cachability from "++show c++" to "++show new_c
+                     when (u `elemQ` w && priority r == High) $ do
+                       modify (\s -> s { waitToStart = pushQ u (deleteQ u w) })
+                       dbg $ "Moving "++u++" to head of download queue."
+                     if f `notElem` (f':fs')
+                        then let new_p = Map.insert u (f', f:fs', new_c, v) p
+                             in do modify (\s -> s { inProgress = new_p })
+                                   dbg "Adding new file to existing UrlRequest."
+                        else dbg "Ignoring UrlRequest of file that's already queued."
+                   _ -> put new_st
           alreadyDownloaded u = do
             n <- liftIO $ withMVar urlNotifications (return . (Map.lookup u))
             case n of
@@ -180,7 +189,7 @@ checkWaitToStart = do
                      , pipeLength = l + 1 }
             dbg ("URL.request_url ("++u++"\n"++
                  "              -> "++f++")")
-            let f_new = f++"-new"
+            let f_new = f++"-new_"++randomJunk st
             liftIO $ do err <- request_url u f_new c
                         if null err
                            then do atexit $ removeFileMayNotExist f_new
@@ -220,22 +229,22 @@ waitNextUrl = do
                 liftIO $ if null e
                          then case Map.lookup u p of
                                 Just (f, fs, _, v) -> do
-                                  renameFile (f++"-new") f
-                                  mapM_ (safeCopyFile f) fs
+                                  renameFile (f++"-new_"++randomJunk st) f
+                                  mapM_ (safeCopyFile st f) fs
                                   putMVar v e
                                   debugMessage $ "URL.waitNextUrl succeeded: "++u++" "++f
                                 Nothing -> bug $ "Possible bug in URL.waitNextUrl: "++u
                          else case Map.lookup u p of
                                 Just (f, _, _, v) -> do
-                                  removeFileMayNotExist (f++"-new")
+                                  removeFileMayNotExist (f++"-new_"++randomJunk st)
                                   putMVar v e
                                   debugMessage $ "URL.waitNextUrl failed: "++
                                                u++" "++f++" "++e
                                 Nothing -> bug $ "Another possible bug in URL.waitNextUrl: "++u++" "++e
                 unless (null u) $ put new_st
-    where safeCopyFile f t = let new_t = t++"-new"
-                             in do copyFile f new_t
-                                   renameFile new_t t
+    where safeCopyFile st f t = let new_t = t++"-new_"++randomJunk st
+                                in do copyFile f new_t
+                                      renameFile new_t t
 
 waitUrl :: String -> IO ()
 waitUrl u = do debugMessage $ "URL.waitUrl "++u
@@ -250,6 +259,14 @@ waitUrl u = do debugMessage $ "URL.waitUrl "++u
 dbg :: String -> StateT a IO ()
 dbg = liftIO . debugMessage
 
+minCachable :: Cachable -> Cachable -> Cachable
+minCachable Uncachable _          = Uncachable
+minCachable _          Uncachable = Uncachable
+minCachable (MaxAge a) (MaxAge b) = MaxAge $ min a b
+minCachable (MaxAge a) _          = MaxAge a
+minCachable _          (MaxAge b) = MaxAge b
+minCachable _          _          = Cachable
+
 #if defined(HAVE_CURL) || defined(HAVE_LIBWWW)
 cachableToInt :: Cachable -> CInt
 cachableToInt Cachable = -1
