@@ -16,7 +16,6 @@
 
 module Main (main) where
 
-import Autoconf ( datadir )
 import Darcs.Commands
 import Darcs.TheCommands
 import Time
@@ -91,7 +90,7 @@ man_header = do
    "\n"++
    "darcs is a nifty revision control tool.  For more a detailed description,\n"++
    "see the html documentation, which should be available at\n"++
-   datadir++"/doc/darcs/manual/index.html.  To easily get more specific help\n"++
+   "/usr/share/doc/darcs/manual/index.html.  To easily get more specific help\n"++
    "on each command, you can call `darcs COMMAND --help'.\n"++
    "\n"
 
