@@ -5,8 +5,8 @@ int wfindps_helper(char c, const char *s, int len);
 void debug_free(void *p);
 void debug_alloc(void *p, const char *name);
 
-int first_white(const char *s, int len);
-int first_nonwhite(const char *s, int len);
+// int first_white(const char *s, int len);
+// int first_nonwhite(const char *s, int len);
 int has_funky_char(const char *s, int len);
 
 char *my_mmap(size_t len, int fd);
