@@ -25,7 +25,6 @@ import Darcs.Ordered ( mapRL, concatRL )
 
 import Data.List ( sort, group )
 import System ( getProgName, getArgs )
-import Control.Monad ( liftM )
 
 intro :: Bool -> IO String
 intro do_stats = do
@@ -44,7 +43,7 @@ intro do_stats = do
 main :: IO ()
 main = withRepository [] $- \repository ->
        do darcs_history <- read_repo repository
-          use_statistics <- elem "stats" `liftM` getArgs
+          use_statistics <- elem "stats" `fmap` getArgs
           intro use_statistics >>= putStr
           mapM_ putStrLn $ sort_authors use_statistics $ mapRL (pi_author.info)
                          $ concatRL darcs_history
@@ -57,9 +56,9 @@ gwern_branwen,
   marnix_klooster, eric_kow, andres_loeh,
   simon_marlow, pekka_pessi,
   erik_schnetter, edwin_thomson, mark_stosberg,
-  thomas_zander, tomasz_zielonka, lele_gaifax :: String
+  thomas_zander, tomasz_zielonka, lele_gaifax , christian_kellermann :: String
 marnix_klooster = "Marnix Klooster <marnix.klooster@gmail.com>"
-eric_kow        = "Eric Kow <eric.kow@gmail.com>"
+eric_kow        = "Eric Kow <kowey@darcs.net>"
 andres_loeh     = "Andres Loeh <mail@andres-loeh.de>"
 simon_marlow    = "Simon Marlow <simonmar@microsoft.com>"
 erik_schnetter  = "Erik Schnetter <schnetter@cct.lsu.edu>"
@@ -70,6 +69,7 @@ edwin_thomson   = "Edwin Thomson <edwin.thomson@businesswebsoftware.com>"
 thomas_zander   = "Thomas Zander <zander@kde.org>"
 tomasz_zielonka = "Tomasz Zielonka <tomasz.zielonka@gmail.com>"
 lele_gaifax     = "Lele Gaifax <lele@nautilus.homeip.net>"
+christian_kellermann = "Christian Kellermann <Christian.Kellermann@nefkom.net>"
 
 -- contributers who only provided an email address, one for which we know the name
 -- note that having an entry in this table counts as two addresses... so if there 
@@ -154,6 +154,7 @@ canonize_author "Andres Loeh <loeh@iai.uni-bonn.de>" = andres_loeh
 canonize_author "benjamin.franksen@bessy.de" = "Benjamin Franksen <ben.franksen@online.de>"
 canonize_author "Lennart Kolmodin <kolmodin@dtek.chalmers.se>" = "Lennart Kolmodin <kolmodin@gentoo.org>"
 canonize_author "kow@loria.fr" = eric_kow
+canonize_author "Eric Kow <eric.kow@gmail.com>" = eric_kow
 canonize_author "Eric Kow <eric.kow@loria.fr>" = eric_kow
 canonize_author "Eric Kow <E.Y.Kow@brighton.ac.uk>" = eric_kow
 canonize_author "schnetter@aei.mpg.de" = erik_schnetter
@@ -185,6 +186,7 @@ canonize_author s
     | s `contains` "mirian@" = "Mirian Crzig Lennox <mirian@cosmic.com>"
     | s `contains` "zbrown" = "Zack Brown <zbrown@tumblerings.org>"
     | s `contains` "daniel.buenzli@epfl.ch" = "Daniel Buenzli <daniel.buenzli@epfl.ch>" -- avoid an accent
+    | s `contains` "Kellermann" = christian_kellermann
 canonize_author s =
   if (not.null) eaLst 
       then (uncurry add_name_to_mail) $ head eaLst
