@@ -4,11 +4,11 @@ RESULT=tmpfile
 ROOT=..
 ERRORSTATUS=0
 
-# lookfor ( $1=what, $2=reason )
+# lookfor ( $1=what, $2=reason, $3=source module exception )
 lookfor () {
     rm -f "$RESULT"
     darcs query manifest --repodir="$ROOT" | grep '\.l\?hs$' | while read f; do
-        grep -FHnwe "$1" "$ROOT/$f" | \
+        grep -Hnwe "$1" "$ROOT/$f" | grep -v "$3\.$1" | \
         grep -v ":[0-9]\+:import " | grep -Fv "ratify $1: " >> "$RESULT"
     done
     if [ -s "$RESULT" ]; then
@@ -25,7 +25,8 @@ lookfor () {
 
 
 lookfor readFile \
-        "readFile doesn't ensure the file is closed before it is deleted!"
+        "readFile doesn't ensure the file is closed before it is deleted!" \
+        B # importing readFile from Data.ByteString as B, is allowed
 
 lookfor hGetContents \
         "hGetContents doesn't ensure the file is closed before it is deleted!"
