@@ -1,9 +1,7 @@
 #!/usr/bin/env bash
 
-set -ev
 
-# The builtin ! has the wrong semantics for not.
-not () { "$@" && exit 1 || :; }
+. lib
 
 # pull from not empty repo to empty repo
 rm -rf temp1 temp2
