@@ -1,21 +1,6 @@
 #!/usr/bin/env bash
 
-set -ev;
-
-## I would use the builtin !, but that has the wrong semantics.
-not () { "$@" && exit 1 || :; }
-
-# darcs does not support cygwin paths (/cygdrive/c/foo), so if
-# we want to run the test suite under Cygwin Bash, we must
-# convert accordingly
-portable_pwd () {
-  if uname | grep -qi cygwin ; then
-    cygpath -w `pwd`
-  else
-    pwd
-  fi
-}
-
+. lib
 
 rm -rf temp1 temp2
 
@@ -69,7 +54,7 @@ not grep temp1 _darcs/prefs/defaultrepo
 
 #return special message when you try to pull from yourself
 IFS=""
-DIR=`portable_pwd`
+DIR=`hspwd`
 not darcs pull --debug -a "$DIR" 2> out
 cat out
 grep 'Can.t pull from current repository' out
@@ -145,7 +130,7 @@ cd ..
 rm -rf temp1 temp2
 
 
-# A test for issue662, which triggered: 
+# A test for issue662, which triggered:
 #  darcs failed:  Error applying hunk to file ./t.t
 #  Error applying patch to the working directory.
 
@@ -166,4 +151,5 @@ darcs pull -a ../
 darcs w -s
 darcs revert -a
 
+cd ../..
 rm -rf tmp
