@@ -1,11 +1,9 @@
 #!/usr/bin/env bash
 ### http://bugs.darcs.net/issue496
 ### _darcs/prefs/defaults ignored when using --repodir
-set -ev
 
 ## All these commands SHOULD fail (hence leading NOTs).
-## I would use the builtin !, but that has the wrong semantics.
-not () { "$@" && exit 1 || :; }
+. lib
 
 rm -rf temp
 mkdir temp
